﻿using ProjectElectricLocomotive.Entities;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Net.NetworkInformation;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace ProjectElectricLocomotive.DrawningObjects
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawningElectricLocomotive : DrawningLocomotive
    {
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="horns">Признак наличия рогов</param>
        /// <param name="battery">Признак наличия отсека для батарей</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public DrawningElectricLocomotive(int speed, double weight, Color bodyColor,
            Color additionalColor, bool horns, bool battery, int width, int height) :
            base(speed, weight, bodyColor, width, height, 120, 70)
        {
            if (EntityLocomotive != null)
            {
                EntityLocomotive = new EntityElectricLocomotive(speed, weight, bodyColor, additionalColor, horns, battery);
            }
        }
        public override void DrawTransport(Graphics g)
        {
            if (EntityLocomotive is not EntityElectricLocomotive electricLocomotive)
            {
                return;
            }
            base.DrawTransport(g);
            Pen pen = new(Color.Black, 5);
            Brush brush = new SolidBrush(EntityLocomotive.BodyColor);
            Point[] points;
            // рога
            if (electricLocomotive.Horns)
            {
                pen = new(Color.Black, 2);
                points = new Point[4];
                points[0] = new Point(_startPosX + 50, _startPosY + 20);
                points[1] = new Point(_startPosX + 40, _startPosY + 10);
                points[2] = new Point(_startPosX + 50, _startPosY);
                points[3] = new Point(_startPosX + 60, _startPosY + 10);
                g.DrawPolygon(pen, points);
            }
            // отсек для батарей
            if (electricLocomotive.Battery)
            {
                brush = new SolidBrush(electricLocomotive.AdditionalColor);
                g.FillRectangle(brush, _startPosX + 80, _startPosY + 45, 35, 9);
            }
        }
    }
}
