﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectElectricLocomotive.Generics
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetGeneric<T>
    where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly T?[] _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Length;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetGeneric(int count)
        {
            _places = new T?[count];
        }

        /// <summary>
        /// обмен двух элементов
        /// </summary>
        /// <param name="first">Первый элемент</param>
        /// <param name="second">Второй элемент</param>
        /// <returns></returns>
        private void swap(ref T first, ref T second)
        {
            T temp = first;
            first = second;
            second = temp;
        }

        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="locomotive">Добавляемый локомотив</param>
        /// <returns></returns>
        public bool Insert(T locomotive)
        {
            int end = _places.Length;
            for (int i = 0; i < _places.Length; ++i)
            {
                if (_places[i] == null)
                {
                    end = i;
                    break;
                }
            }
            if (end == _places.Length)
            {
                return false;
            }
            for (int i = end; i >= 1; --i)
            {
                _places[i] = _places[i - 1];
            }
            _places[0] = locomotive;
            return true;
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="locomotive">Добавляемый поезд</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public bool Insert(T locomotive, int position)
        {
            if (position >= _places.Length || position < 0)
            {
                return false;
            }
            if (_places[position] == null)
            {
                _places[position] = locomotive;
                return true;
            }
            int end = _places.Length;
            for (int i = position; i < _places.Length; ++i)
            {
                if (_places[i] == null)
                {
                    end = i;
                    break;
                }
            }
            if (end == _places.Length)
            {
                return false;
            }
            _places[end] = locomotive;
            for (int i = position; i < end; ++i)
            {
                swap(ref _places[i], ref _places[end]);
            }
            return true;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public bool Remove(int position)
        {
            if (position >= _places.Length || position < 0)
            {
                return false;
            }

            _places[position] = null;
            return true;

        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T? Get(int position)
        {
            if (position >= _places.Length || position < 0)
            {
                return null;
            }
            return _places[position];
        }

    }
}