﻿using ProjectElectricLocomotive.DrawningObjects;
using ProjectElectricLocomotive.MovementStrategy;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectElectricLocomotive
{
    /// <summary>
    /// Форма работы с объектом "электровоз"
    /// </summary>
    public partial class FormLocomotive
    {
        /// <summary>
        /// Поле-объект для прорисовки объекта
        /// <summary>
        private DrawningLocomotive? _drawningLocomotive;
        /// <summary>
        /// Стратегия перемещения
        /// </summary>
        private AbstractStrategy? _strategy;
        /// <summary>
        /// Выбранный электровоз
        /// </summary>
        public DrawningLocomotive? SelectedLocomotive { get; private set; }
        /// <summary>
        /// Инициализация формы
        /// </summary>
        public FormLocomotive()
        {
            InitializeComponent();
            _strategy = null;
            SelectedLocomotive = null;
        }
        /// <summary>
        /// Метод прорисовки локомотива
        /// </summary>
        private void Draw()
        {
            if (_drawningLocomotive == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxElectricLocomotive.Width,
            pictureBoxElectricLocomotive.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningLocomotive.DrawTransport(gr);
            pictureBoxElectricLocomotive.Image = bmp;
        }

        /// Обработка нажатия кнопки "Создать электровоз"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreateElectricLocomotive_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color bodyColor = Color.FromArgb(
                random.Next(0, 256),
                random.Next(0, 256),
                random.Next(0, 256));
            Color additionalColor = Color.FromArgb(
                random.Next(0, 256),
                random.Next(0, 256),
                random.Next(0, 256));
            ColorDialog dialog;
            dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                bodyColor = dialog.Color;
            }
            dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                additionalColor = dialog.Color;
            }
            _drawningLocomotive = new DrawningElectricLocomotive(
                random.Next(100, 300),
                random.Next(1000, 3000),
                bodyColor,
                additionalColor,
                Convert.ToBoolean(random.Next(0, 2)),
                Convert.ToBoolean(random.Next(0, 2)),
                pictureBoxElectricLocomotive.Width, pictureBoxElectricLocomotive.Height);
            _drawningLocomotive.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        /// <summary>
        /// Обработка нажатия кнопки "Создать локомотив"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreateLocomotive_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(
                random.Next(0, 256),
                random.Next(0, 256), 
                random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }
            _drawningLocomotive = new DrawningLocomotive(
                random.Next(100, 300),
                random.Next(1000, 3000),
                color,
                pictureBoxElectricLocomotive.Width, pictureBoxElectricLocomotive.Height);
            _drawningLocomotive.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        /// <summary>
        /// Изменение размеров формы
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PictureBoxElectricLocomotive_SizeChanged(object sender, EventArgs e)
        {
            Size size = ((PictureBox)sender)?.Size ?? Size.Empty;
            
            if (_strategy != null)
            {
                _strategy.SetFieldSize(size);
            }
            if (_drawningLocomotive != null)
            {
                _drawningLocomotive.SetPictureSize(size);
            }
        }

        /// <summary>
        /// Изменение положения автомобиля
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawningLocomotive == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawningLocomotive.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    _drawningLocomotive.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    _drawningLocomotive.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    _drawningLocomotive.MoveTransport(DirectionType.Right);
                    break;
            }
            Draw();
        }

        /// <summary>
        /// Шаг стратегии перемещения
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonnStartegyStep_Click(object sender, EventArgs e)
        {
            if (_drawningLocomotive == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _strategy = comboBoxStrategy.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_strategy == null)
                {
                    return;
                }
                _strategy.SetData(
                    _drawningLocomotive.GetMoveableObject,
                    pictureBoxElectricLocomotive.Width,
                    pictureBoxElectricLocomotive.Height
                );
            }
            if (_strategy == null)
            {
                return;
            }
            comboBoxStrategy.Enabled = false;
            _strategy.MakeStep();
            Draw();
            if (_strategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _strategy = null;
            }
        }

        /// <summary>
        /// Выбор локомотива
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSelectLocomotive_Click(object sender, EventArgs e)
        {
            SelectedLocomotive = _drawningLocomotive;
            DialogResult = DialogResult.OK;
        }
    }
}
