﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectElectricLocomotive.DrawningObjects;
using ProjectElectricLocomotive.Generics;
using ProjectElectricLocomotive.MovementStrategy;

namespace ProjectElectricLocomotive
{
    /// <summary>
    /// Форма для работы с набором объектов класса DrawningLocomotive
    /// </summary>

    public partial class FormLocomotiveCollection : Form
    {
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly LocomotivesGenericCollection<
            DrawningLocomotive, DrawningObjectLocomotive> _locomotives;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormLocomotiveCollection()
        {
            InitializeComponent();
            _locomotives = new LocomotivesGenericCollection<DrawningLocomotive, DrawningObjectLocomotive>(
                pictureBoxCollection.Width, pictureBoxCollection.Height
                );
        }

        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddLocomotive_Click(object sender, EventArgs e)
        {
            FormLocomotive form = new();
            if (form.ShowDialog() == DialogResult.OK)
            {
                if (_locomotives + form.SelectedLocomotive > 0)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBoxCollection.Image = _locomotives.ShowLocomotives();
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            }
        }
        /// <summary>
        /// Удаление объекта из набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveLocomotive_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(
                "Удалить объект?", "Удаление",
                MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxNumber.Text);
            if (_locomotives - pos != -1)
            {
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = _locomotives.ShowLocomotives();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        /// <summary>
        /// Обновление рисунка по набору
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRefreshCollection_Click(object sender, EventArgs e)
        {
            pictureBoxCollection.Image = _locomotives.ShowLocomotives();
        }
    }
}
